// DRO cyfrowe wersja 1.1
//
// Wyswietlacz 8 cyfrowy, LED 7-segmentowy, sterownik 74HC595
// Podlaczenie pinow wyswietlacza:
// DIO -- 2
// SCK -- 3
// RCK -- 4
// GND -- GND
// VCC -- 5V

//poczatek konfiguracji

// liczba całkowita - odleglosc na impuls w 0,000001mm, max 32000
int distance = 1000;
// liczba całkowita - ilosc impulsow dla luzu w nakretce
int clearance = 50;
int DIOpin = 2;
int SCKpin = 3;
int RCKpin = 4;
int resetPin = 6;
int EncApin = 8;
int EncBpin = 9;

// koniec konfiguracji

byte digitData = 0;
byte digitAddress = 0;
int displayRound = 0;
int timer = 0;
int displayValue[8] = {0,0,0,0,0,0,0,0};
byte navigation[4][2] = {{0,0},{1,0},{1,1},{0,1}};
long result;
int clearanceForward = 0;
int clearanceBack;
int next = 2;
int back = 0;
long absResult = 0;
long modulo = 0;
byte zero;

void setup()
{
  pinMode(DIOpin, OUTPUT);
  pinMode(RCKpin, OUTPUT);
  pinMode(SCKpin, OUTPUT);
  pinMode(EncApin, INPUT_PULLUP);
  pinMode(EncBpin, INPUT_PULLUP);
  pinMode(resetPin, INPUT_PULLUP);
  
  clearanceBack = clearance;
  resetPosition();
  
}


void loop()
{
  if(!digitalRead(resetPin)) resetPosition();
  
  if(digitalRead(EncApin) == navigation[next][0] && digitalRead(EncBpin) == navigation[next][1]) {
    if(next < 3) next++;
    else next = 0;
    if(back < 3) back++;
    else back = 0;
    result += moveForward();
  } else if(digitalRead(EncApin) == navigation[back][0] && digitalRead(EncBpin) == navigation[back][1]) {
    if(next > 0) next--;
    else next = 3;
    if(back > 0) back--;
    else back = 3;
    result -= moveBack();
  }
  
  refreshDisplay();
}

void resetPosition() {
  result = 0;
  clearanceForward = 0;
  clearanceBack = clearance;
  
  for(byte i = 0; i <= 3; i++) {
    if(digitalRead(EncApin) == navigation[i][0] && digitalRead(EncBpin) == navigation[i][1]) {
      if(i < 3) next = i+1;
      else next = 0;
      if(i > 0) back = i-1;
      else back = 3;
    }
  }
}

byte moveForward(){
   
  if(clearanceForward == 0) {
    return distance;
  }
  
  clearanceForward -= 1;
  clearanceBack += 1;
   
  return 0;
}

byte moveBack(){
   
  if(clearanceBack == 0) {
    return distance;
  }
  
  clearanceBack -= 1;
  clearanceForward += 1;
   
  return 0;
}

int setNumber(int Num_seg, byte dot = false)
{
  switch (Num_seg)
  {
    case 0: digitData = 0x03; break;
    case 1: digitData = 0x9F; break;
    case 2: digitData = 0x25; break;
    case 3: digitData = 0x0D; break;
    case 4: digitData = 0x99; break;
    case 5: digitData = 0x49; break;
    case 6: digitData = 0x41; break;
    case 7: digitData = 0x1F; break;
    case 8: digitData = 0x01; break;
    case 9: digitData = 0x09; break;
    case 10: digitData = 0xFD; break;
    case 11: digitData = 0xFF; break;
    default: digitData = 0xFF; break;
  }
  
  if(!dot) return digitData;
  else return --digitData;
}

int setDigit(int number)
{
  // kolejnosc 4,3,2,1,8,7,6,5
  
  switch (number)
  {
    case 1: digitAddress = 0b00010000; break;
    case 2: digitAddress = 0b00100000; break;
    case 3: digitAddress = 0b01000000; break;
    case 4: digitAddress = 0b10000000; break;
    case 5: digitAddress = 0b00000001; break;
    case 6: digitAddress = 0b00000010; break;
    case 7: digitAddress = 0b00000100; break;
    case 8: digitAddress = 0b00001000; break;
    default: digitAddress = 0x00; break;
  }
  return digitAddress;
}

void refreshDisplay() {
  
  if(timer == 40) {
   refreshResult();
  } else if(displayRound == 9) {
   digitalWrite(RCKpin, LOW);
   shiftOut(DIOpin, SCKpin, LSBFIRST, setNumber(0));
   shiftOut(DIOpin, SCKpin, MSBFIRST, setDigit(0));
   digitalWrite(RCKpin, HIGH);
   displayRound = 0;
   timer++;
   
  } else {
   digitalWrite(RCKpin, LOW);
   
   if(displayRound != 5) shiftOut(DIOpin, SCKpin, LSBFIRST, setNumber(displayValue[displayRound-1]));
   else shiftOut(DIOpin, SCKpin, LSBFIRST, setNumber(displayValue[displayRound-1], true));
   
   shiftOut(DIOpin, SCKpin, MSBFIRST, setDigit(displayRound));
   digitalWrite(RCKpin, HIGH);
  }  
  displayRound++;
}

void refreshResult() {
  timer = 0;
  zero = false;
  if(result > 0) displayValue[0] = 11;
  else displayValue[0] = 10;
  absResult = abs(result);
  displayValue[1] = absResult / 1000000000;
  if(displayValue[1] == 0) displayValue[1] = 11;
  modulo = absResult % 1000000000;
  displayValue[2] = modulo / 100000000;
  if(displayValue[2] == 0) displayValue[2] = 11;
  modulo = modulo % 100000000;
  displayValue[3] = modulo / 10000000;
  if(displayValue[3] == 0) displayValue[3] = 11;
  modulo = modulo % 10000000;
  displayValue[4] = modulo / 1000000;
  modulo = modulo % 1000000;
  displayValue[5] = modulo / 100000;
  modulo = modulo % 100000;
  displayValue[6] = modulo / 10000;
  modulo = modulo % 10000;
  displayValue[7] = modulo / 1000;
}
